package hogan;

import futils.Futil;
import futils.WriterUtil;
import gui.ClosableJFrame;
import gui.run.*;
import j2d.ImageUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileNotFoundException;

public class AffineImagePanel extends JPanel
        implements MouseMotionListener {
    double theta = 0;
    Point2D location = new Point2D.Float(0, 0);
    Point2D scale = new Point2D.Double(1.0, 1.0);
    Point2D shear = new Point2D.Double(0, 0);

    Point mousePoint = new Point(0, 0);

    private boolean rotateAboutMouse = false;
    private boolean defaultRotateScaleShear = false;

    File f;

    Image img;

    public static void main(String args[]) {
        new AffineImagePanel();
    }

    AffineImagePanel() {
        try {
            img = getImage();
            displayAffineImageFrame();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        addMouseMotionListener(this);
    }

    public static Image getImage() throws FileNotFoundException {
        File f = Futil.getReadFile("Enter a GIF or JPEG");
        return ImageUtils.getImage(f);
    }

    public void displayControlPanel() {
        ClosableJFrame cf = new ClosableJFrame();
        cf.setJMenuBar(getMenuBar());
        Container c = cf.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(getControlPanel(), BorderLayout.EAST);
        cf.setSize(500, 400);
        cf.setLocation(500, 0);
        cf.show();
    }

    public void displayAffineImageFrame() {
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(this, BorderLayout.CENTER);
        displayControlPanel();
        int w = img.getWidth(this);
        int h = img.getHeight(this);
        cf.setSize(w, h);
        cf.show();
    }

    public JMenuBar getMenuBar() {
        JMenuBar mb = new JMenuBar();
        mb.add(getFileMenu());
        mb.add(getAffineMenu());
        return mb;
    }

    public RunMenu getFileMenu() {
        RunMenu rm = new RunMenu("[File");
        rm.add(new RunMenuItem("[Save{ctrl S}") {
            public void run() {
                ImageUtils.saveAsGif(img,
                        WriterUtil.getSaveFileName("Save Image As"));
            }
        });
        rm.add(new RunMenuItem("[Open{ctrl O}") {
            public void run() {
                img = ImageUtils.getImage();
            }
        });
        return rm;
    }

    public RunMenu getAffineMenu() {
        RunMenu rm = new RunMenu("[Affine Selector");
        rm.add(new RunCheckBoxMenuItem("[About Mouse{ctrl A}") {
            public void run() {
                if (!defaultRotateScaleShear)
                    rotateAboutMouse = super.getState();
            }
        });
        rm.add(new RunCheckBoxMenuItem("[Default Rotate Scale Shear{ctrl D}") {
            public void run() {
                if (!rotateAboutMouse)
                    defaultRotateScaleShear = super.getState();
            }
        });
        return rm;
    }

    public JPanel getControlPanel() {
        JPanel jp = new JPanel();
        jp.setLayout(new GridLayout(0, 2));
        jp.add(new RunLabel("Rotate" + " "));
        jp.add(new RunSlider(0, 360, 0) {
            public void run() {
                rotate(getValue());
            }
        });
        jp.add(new RunLabel("Translate X" + " "));
        jp.add(new RunSlider(0, 600, 0) {
            public void run() {
                translateX(getValue());
            }
        });
        jp.add(new RunLabel("Translate Y" + " "));
        jp.add(new RunSlider(0, 600, 0) {
            public void run() {
                translateY(getValue());
            }
        });
        jp.add(new RunLabel("Scale X" + " "));
        jp.add(new RunSlider(0, 200, 100) {
            public void run() {
                scaleX(getValue() / 100.0);
            }
        });
        jp.add(new RunLabel("Scale Y" + " "));
        jp.add(new RunSlider(0, 200, 100) {
            public void run() {
                scaleY(getValue() / 100.0);
            }
        });
        jp.add(new RunLabel("Shear X" + " "));
        jp.add(new RunSlider(-400, 400, 0) {
            public void run() {
                shearX(getValue() / 100.0);
            }
        });
        jp.add(new RunLabel("Shear Y" + " "));
        jp.add(new RunSlider(-400, 400, 0) {
            public void run() {
                shearY(getValue() / 100.0);
            }
        });
        return jp;
    }

    public void translateX(int x) {
        location.setLocation(x, location.getY());
        repaint();
    }

    public void translateY(int y) {
        location.setLocation(location.getX(), y);
        repaint();
    }

    public void scaleX(double sx) {
        scale.setLocation(sx, scale.getY());
        repaint();
    }

    public void scaleY(double sy) {
        scale.setLocation(scale.getX(), sy);
        repaint();
    }

    public void shearX(double sx) {
        shear.setLocation(sx, shear.getY());
        repaint();
    }

    public void shearY(double sy) {
        shear.setLocation(shear.getX(), sy);
        repaint();
    }

    public void rotate(double radians) {
        theta = radians;
        repaint();
    }

    /*
     * Rotate the arrow using degrees for the angle.
     *
     * @param g2
     * @param sx
     * @param sy
     * @param location
     * @param angle
     */

    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Point2D trPt = location;
        double w = img.getWidth(this);
        double h = img.getHeight(this);

        if (rotateAboutMouse) {
            trPt = new Point2D.Double(mousePoint.getX(), mousePoint.getY());
        }
        if (defaultRotateScaleShear) {
            trPt = new Point2D.Double(w / 2.0, h / 2.0);
        }
        drawImage(g2, trPt, scale.getX(), scale.getY(), theta, shear.getX(), shear.getY());
    }

    public void mouseDragged(MouseEvent e) {
        mousePoint = e.getPoint();
        repaint();

    }

    public void mouseMoved(MouseEvent e) {

    }

    private void drawImage(Graphics2D g2, Point2D location, double sx, double sy,
                           double angle, double shx, double shy) {
        Dimension d = getSize();
        g2.fillRect(0, 0, d.width, d.height);
        setUpAffineTransform(g2, location, sx, sy, angle, shx, shy);
        if (img == null) return;
        //g2.drawImage(image, 0, 0, d.width / 2, d.height / 2, this);
        g2.drawImage(img, 0, 0, this);
    }

    private void setUpAffineTransform(Graphics2D g2, Point2D location, double sx, double sy,
                                      double angle, double shx, double shy) {
        AffineTransform at = new AffineTransform();

        at.setToTranslation(location.getX(), location.getY());
        at.shear(shx, shy);
        at.scale(sx, sy);
        at.rotate(angle * Math.PI / 180);//, 10, 10);
        at.translate(-location.getX(), -location.getY());
        g2.setTransform(at);
    }

    private void drawArrow(Graphics2D g2, Point2D location, float sx, float sy,
                           double angle, double shx, double shy) {
        GeneralPath gp = getArrow();
        //erase(g2, gp);
        setUpAffineTransform(g2, location, sx, sy, angle, shx, shy);
        g2.draw(gp);
    }

    private void erase(Graphics2D g2, GeneralPath gp) {
        g2.setXORMode(getBackground());
        g2.draw(gp);
        g2.setXORMode(getForeground());
    }


    private GeneralPath getArrow() {
        GeneralPath gp = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
        gp.moveTo(0f, -15f);
        gp.lineTo(10f, 5f);
        gp.lineTo(5f, 5f);
        gp.lineTo(5f, 15f);
        gp.lineTo(-5, 15f);
        gp.lineTo(-5f, 5f);
        gp.lineTo(-10f, 5f);
        gp.closePath();
        return gp;
    }
}
